package com.prosperolambda.android.app.hecate;

import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

public class GpsControl implements LocationListener, Runnable  {
	
	/* Utiliza patrn Singleton. Solo una instancia por ejecucin.
	 * Implementado segn Efficient Java (pag. 18)
	 */
	private static final GpsControl INSTANCE = new GpsControl();
	private static final String TAG="GpsControl";
	
	/*
	 * Constantes para parmetros de actualizacin de
	 * datos del GPS de forma dinmica.
	 */
	private static final float DISTANCIA_MIN=100; //Distancia mnima entre actualizaciones del gps
	private static final long TIEMPO_MIN=10000;//Tiempo mnimo entre actualizaciones del gps (10 segundos)
	
	private boolean estaArrancado;
	private Location mLastLocation;
	private LocationManager mLocationManager;
	private Thread mThread;
	private Handler mHandler;
	private Context mContext;
	

	private GpsControl(){
		mContext=null;
		mLastLocation=null;
		mLocationManager= null;
		mThread=null;
		mHandler=null;
		estaArrancado=false;
	}
	
	public boolean arrancarGps(){
		if(!estaArrancado){
			//Obtiene el LocationManager a travs del contexto pasado como parmetro.
			try{
				mLocationManager= (LocationManager) mContext.getSystemService(Context.LOCATION_SERVICE);
			}catch(NullPointerException e){
				e.printStackTrace();
				Log.e(TAG, "Contexto no iniciado!");
			}
			//Crea un nuevo hilo y lo arranca.
			mThread=new Thread(this);
			mThread.start();
			
			estaArrancado=true;
		}
		return estaArrancado;
	}
	
	public boolean pararGps(){
		//Deja de recibir actualizaciones del Gps
		if(estaArrancado){
			mLocationManager.removeUpdates(this);
			//Elimina la recepcin de callbacks y la cola de mensajes de este hilo
			( mHandler.getLooper() ).quit();
			estaArrancado=false;
			Log.d(TAG, "Parando GPS y Looper");
		}
		return estaArrancado;
	}
	
	@Override
	public void run(){
		//Prepara Looper para poder recibir Callbacks en este hilo
		Looper.prepare();
		
		//Crea un nuevo Handler asociado al Looper del nuevo hilo.
		//Necesario para tener una referencia a dicho Looper y poder
		//terminarlo ms adelante.
		mHandler=new Handler();
		
		//Registra el listener y empieza a escuchar actualizaciones del GPS
		mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
				TIEMPO_MIN, DISTANCIA_MIN, this, Looper.myLooper() );
		
		//Empieza a escuchar Callbacks en este hilo
		Looper.loop();
		Log.d(TAG, "Nuevo hilo creado, con Looper propio");
	}
	
	public static GpsControl getInstance(){
		return INSTANCE; //Patrn Singleton
	}
	
	public void setContext(Context e){
		mContext=e;
	}

	@Override
	public synchronized void onLocationChanged(Location location) { 
		//Tan solo actualiza la referencia del objeto con los nuevos datos.
		Log.d(TAG, "Nueva posicin de GPS recibida");
		mLastLocation=location;
	}

	public synchronized Location getUltimaLocalizacion(){
		//Devuelve la ltima versin del objeto Location.
		Log.d(TAG, "Devolviendo objeto Location");
		return mLastLocation;
	}
	
	public synchronized double[] getPosicion(){
		//Devuelve la ltima posicin recibida en un array con latitud y longitud.
		Log.d(TAG, "Devolviendo Latitud y Longitud");
		double [] resultado= new double[2];
		resultado[0]=(mLastLocation != null)? mLastLocation.getLatitude() : 0.0;
		resultado[1]=(mLastLocation != null)? mLastLocation.getLongitude() : 0.0;
		
		return resultado;		
	}
	
	public synchronized float getVelocidad(){
		//Devuelve la velocidad experimentada en la ltima recepcin del gps
		Log.d(TAG, "Devolviendo velocidad");
		return (mLastLocation != null) ? mLastLocation.getSpeed() : 0.0f;
	}
	
	public synchronized float getPrecision(){
		//Devuelve la precisin actual del GPS
		Log.d(TAG, "Devolviendo precisin");
		return (mLastLocation != null) ? mLastLocation.getAccuracy() : 0.0f;
	}
	
	//No necesario para el uso actual del gps
	@Override
	public void onProviderDisabled(String provider) { }

	//No necesario para el uso actual del gps
	@Override
	public void onProviderEnabled(String provider) { }

	//No necesario para el uso actual del gps
	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) { }
}
