package com.prosperolambda.android.app.hecate;

import java.util.Arrays;

import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.IBinder;
import android.os.PowerManager;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.widget.Toast;

public class MonitorService extends Service implements SensorEventListener{
	
	private Sensor mSensor;	
	private SensorManager mSensorManager;
	
	private PowerManager mPowerManager;
	private PowerManager.WakeLock mWakelock;
	private GpsControl mGps;
	private boolean mbModoDebug;
	
	private float [][] mfCola;
	private int miIndiceCola;
	private int miContador;	
	private float mfUmbral;
	
	private static boolean mbActivo=false;
	
	private static final String WAKELOCK_TAG="MonitorService_Wakelock"; //Etiqueta para Log
	private static final String TAG="MonitorService"; //Etiqueta para Log
	private static final int ID=1234; //Identificador nico de Servicio	
	/*
	 * En la memoria se justifica el valor de 4 como umbral mximo de aceleracin por
	 * traza permitido.
	 */
	private static final float UMBRAL= 4f * SensorManager.GRAVITY_EARTH;
	private static final int TAM_COLA=10; //Tamao de la cola circular
	/*
	 * Nmero mximo de trazas consetuvicas que pueden superar el umbral de aceleracin
	 * sin hacer saltar la alarma.
	 */
	private static final int MAX_TRAZAS=6;
	private static final float ALFA=0.4f; //Parmetro del filtro paso bajo
	
	@Override
	public void onCreate(){
		super.onCreate();
		//Obtiene el Acelerermetro lineal. Se ha comprado previamente que puede usarse.
		mSensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
		mSensor= mSensorManager.getDefaultSensor(Sensor.TYPE_LINEAR_ACCELERATION);
		
		/*
		 * Es necesario adquirir un Wakelock parcial, de lo contrario el dispositivo
		 * podra entrar en modo "deep sleep".
		 */
		mPowerManager= (PowerManager) getSystemService(Context.POWER_SERVICE);		
		mWakelock= mPowerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, WAKELOCK_TAG);
		
		/*
		 * Establece el umbral de aceleracin mxima dependiendo de si el Modo Debug (1G)
		 * est activado o no.
		 */
		SharedPreferences preferencias = PreferenceManager.getDefaultSharedPreferences(this);
		mbModoDebug= preferencias.getBoolean("opciones_debug", false);
		Log.d(TAG, "Modo debug = "+mbModoDebug );
		
		mfUmbral = mbModoDebug ? SensorManager.GRAVITY_EARTH : UMBRAL;
		
		//Obtiene una instancia de GpsControl
		mGps= GpsControl.getInstance();
		
		//Crea las colas circulares e inicializa los ndices y flags
		mfCola = new float[3][TAM_COLA]; // 3 ejes (x,y,z) de 10 elementos cada uno
		Arrays.fill(mfCola[0], 0.0f);
		Arrays.fill(mfCola[1], 0.0f);
		Arrays.fill(mfCola[2], 0.0f);//Rellena el array con ceros
		miIndiceCola=0;
		miContador=0;
		
		Log.d(TAG, "Servicio CREADO");
	}
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startld){
		mbActivo=true;
		/* Hasta desactivar el wakelock, la CPU permanecer activa, evitando
		 * que el S.O. pueda poner a dormir la aplicacin.
		 */
		mWakelock.acquire();
		
		//Empieza a recibir datos del sensor
		mSensorManager.registerListener(this, mSensor, SensorManager.SENSOR_DELAY_UI);
		Log.d(TAG, "Acelermetro ESCUCHANDO");
		
		//GPS empieza a actualizar la posicin.
		mGps.arrancarGps();
		Log.d(TAG, "Gps ESCUCHANDO");
		
		Toast toast= Toast.makeText( getApplicationContext(), R.string.serviciomonitor_toast_iniciado, Toast.LENGTH_LONG);
		toast.show();
		Log.d(TAG, "Servicio INICIADO");
		arrancarForeground();
		Log.d(TAG, "Servicio en FOREGROUND");
		return START_REDELIVER_INTENT;
	}
	
	@Override
	public void onDestroy(){
		super.onDestroy();
		//El servicio dejar de funcionar, se cambia la variable esttica.
		mbActivo=false;
		//Se "desregistra" el listener del sensor (muy importante!)
		mSensorManager.unregisterListener(this);
		//Controlador del Gps deja de recibir datos.
		mGps.pararGps();
		//Por ltimo se devuelve el testigo del Wakelock		
		mWakelock.release();
		Log.d(TAG, "Partial_Wakelock LIBERADO");
		
		Toast toast= Toast.makeText( getApplicationContext(), R.string.serviciomonitor_toast_detenido, Toast.LENGTH_LONG);
		toast.show();
		Log.d(TAG, "Servicio DESTRUDO");
	}
	
	private void arrancarForeground(){
		/*
		 * Primero se crea un Intent para determinar que accin realiza la 
		 * notificacin cuando sea pulsada. En este caso vuelve a abrir
		 * MainActivity.
		 * 
		 * Se crea con el Intent anterior un PendingIntent que ser el usado por
		 * la notificacin.
		 */
		Intent intentNotification=new Intent(this, MainActivity.class);
		PendingIntent pendingIntent= PendingIntent.getActivity(this, 0, intentNotification, 0);
		
		/*
		 * Notification est deprecado y se recomienda usar NotificationCompat.Builder.
		 * Se aade el icono, ttulo, texto y se aade el PendingIntent
		 */
		NotificationCompat.Builder notificationBuilder= new NotificationCompat.Builder(this)
				.setSmallIcon(R.drawable.ic_servicio_monitor)
				.setContentTitle( getText(R.string.notificacion_titulo) )
				.setContentText( getText(R.string.notificacion_texto) )
				.setContentIntent(pendingIntent);
		//Por ltimo se construye la notificacin.
		Notification notification= notificationBuilder.build();
		//Se establece el servicio en "foreground".
		startForeground(ID, notification);				
		
	}
	
	//No usado en este tipo de servicio
	@Override
	public IBinder onBind(Intent arg0) {
		return null;
	}

	public static boolean estaActivo(){
		return mbActivo;
	}

	@Override
	public void onSensorChanged(SensorEvent event) {
		procesarTraza(event.values);
	}
	
	private void procesarTraza(float[] traza){
//		Log.d(TAG, "Procesando traza");
		int actual= miIndiceCola;
		//Si el ndice est en 0, el anterior corresponde a TAM_COLA-1
		int anterior = miIndiceCola>0 ? miIndiceCola-1 : TAM_COLA-1; 
		
		//Primero se aplica un filtro paso-bajo a los valores de cada eje en su cola correspondiente
		//Siempre en valores absolutos
		mfCola[0][actual]= mfCola[0][anterior] + ( Math.abs(traza[0]) - mfCola[0][anterior] )*ALFA;
		mfCola[1][actual]= mfCola[1][anterior] + ( Math.abs(traza[1]) - mfCola[1][anterior] )*ALFA;
		mfCola[2][actual]= mfCola[2][anterior] + ( Math.abs(traza[2]) - mfCola[2][anterior] )*ALFA;
		
//		Log.d(TAG, "Aceleraciones ["+traza[0]+" "+traza[1]+" "+traza[2]+"]");
//		Log.d(TAG, "Filtrado ["+mfCola[0][actual]+" "
//				+mfCola[1][actual]+" "+mfCola[2][actual]+"]");
		
		miIndiceCola = (miIndiceCola +1) % TAM_COLA;
		
		//Si cualquiera de los valores filtrados supera el umbral...
		if( (mfCola[0][actual]>=mfUmbral) || 
				(mfCola[1][actual]>=mfUmbral) || 
					(mfCola[2][actual]>=mfUmbral) ){
			
			Log.d(TAG, "ACELERACION Mxima Superada. Contador= "+miContador);
			//Incrementa el contador y comprueba si supera el lmite permitido de trazas
			if(++miContador>MAX_TRAZAS)
				lanzarAlarma(); //Lanza la Actividad de Alarma al usuario.			
		}
		else
			miContador=0;
		return;
	}

	
	private void lanzarAlarma(){
		Intent intent=new Intent(this, AlarmaActivity.class);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		
		startActivity(intent);
		stopSelf();
	}
	
	//No necesario para el uso actual del Acelermetro lineal.
	@Override
	public void onAccuracyChanged(Sensor arg0, int arg1) { }
	
}
